@@ -5,6 +5,7 @@ import cfg4py
 import fire
 import omicron
 from omicron.models.timeframe import TimeFrame
+from pyemit import emit
 from sanic import Sanic
 
 from backtest.config import get_config_dir
@@ -16,16 +17,32 @@ application = Sanic("backtest")
 logger = logging.getLogger(__name__)
 
 
+@application.listener("before_server_start")
 async def application_init(app, *args):
-    logger.info("init backtest server")
     try:
         await omicron.init()
     except Exception:
+        logger.warning(
+            "omicron running in degrade mode, this may cause inaccurate results due to calendar issues"
+        )
         TimeFrame.service_degrade()
 
+    cfg = cfg4py.get_instance()
+    await emit.start(emit.Engine.REDIS, start_server=True, dsn=cfg.redis.dsn)
     feed = await BaseFeed.create_instance(interface="zillionare")
+    await feed.init()
+
     app.ctx.feed = feed
     app.ctx.accounts = Accounts()
+    app.ctx.accounts.on_startup()
+
+
+@application.listener("after_server_stop")
+async def application_exit(app, *args):
+    await omicron.close()
+    await emit.stop()
+    accounts = app.ctx.accounts
+    accounts.on_exit()
 
 
 def start(port: int):
@@ -38,7 +55,6 @@ def start(port: int):
 
     application.blueprint(bp)
 
-    application.register_listener(application_init, "before_server_start")
     application.run(host="0.0.0.0", port=port, register_sys_signals=True)
     logger.info("backtest server stopped")
 
