@@ -6,10 +6,7 @@ class GenericErrCode(IntEnum):
     UNKNOWN = -1
 
     def __str__(self):
-        return {
-            GenericErrCode.OK: "成功",
-            GenericErrCode.UNKNOWN: "失败",
-        }.get(self)
+        return {GenericErrCode.OK: "成功", GenericErrCode.UNKNOWN: "失败"}.get(self)
 
 
 class Error(Exception):
@@ -33,3 +30,9 @@ class NoDataForMatchError(Error):
     """缺少撮合数据"""
 
     pass
+
+
+class AccountError(Error):
+    """账户冲突，或者已冻结"""
+
+    pass
