@@ -30,18 +30,15 @@ class Config(object):
     class server:
         path: Optional[str] = None
 
-        port: Optional[int] = None
+    class auth:
+        admin: Optional[str] = None
 
-    accounts: Optional[list] = None
+    class feed:
+        type: Optional[str] = None
 
     class redis:
         dsn: Optional[str] = None
 
-    class postgres:
-        dsn: Optional[str] = None
-
-        enabled: Optional[bool] = None
-
     class influxdb:
         url: Optional[str] = None
 
@@ -52,3 +49,8 @@ class Config(object):
         bucket_name: Optional[str] = None
 
         enable_compress: Optional[bool] = None
+
+    class postgres:
+        dsn: Optional[str] = None
+
+        enabled: Optional[bool] = None
